/************************************************************************
	main.c

	WFF USB Generic HID Demonstration 3
    usbGenericHidCommunication reference firmware 3_0_0_0
    Copyright (C) 2011 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#ifndef MAIN_C
#define MAIN_C

// if running on the simulator, the simulator will get stuck in the SPI write code.
// defining debug removes the SPI write code.
//#define DEBUG

// changes for the 4x4 square
//#define S4X4

// Global includes
// Note: string.h is required for sprintf commands for debug
#include <string.h>

// Local includes
#include "HardwareProfile.h"  
#include "debug.h"

// Microchip Application Library includes
// (expects V2.9a of the USB library from "Microchip Solutions v2013-02-15")
//
// The library location must be set in:
// Project -> Build Options Project -> Directories -> Include search path
// in order for the project to compile.
#include "./USB/usb.h"
#include "./USB/usb_function_hid.h"

// SPI libs
#include <spi.h>  //Include C18 SPI Library
#include <delays.h> //delay Library
#include <timers.h>
#include <EEP.H>

#define table_count 14   //bad Dwayne has 2nd copy of this in patterns_4x4x4.c
extern rom unsigned char *patterns[];
extern rom unsigned int pattern_size[];
extern rom unsigned char pattern_demo_loops[];

// Ensure we have the correct target PIC device family
#if !defined(__18F4550) && !defined(__18F2550) && !defined(__18F2450) && !defined(__18F25K50)
	#error "This firmware only supports either the PIC18F4550 or PIC18F2550 microcontrollers."
#endif

// Define the globals for the USB data in the USB RAM of the PIC18F*550
#pragma udata
#pragma udata USB_VARIABLES=0x500 
unsigned char ReceivedDataBuffer[64];
unsigned char ToSendDataBuffer[64];
#pragma udata

typedef struct {
   unsigned char red;
   unsigned char green;
   unsigned char blue;
} strobes;

#pragma udata
#pragma udata LED_VARIABLES=0x200 
strobes led_data[64];
strobes led_data_save[5];
#pragma udata
unsigned char red_out=0, green_out=0, blue_out=0, wait_timer = 1;
unsigned char strobe = 0;
unsigned char intensity = 0;
unsigned char idelay = 0;
#pragma udata

USB_HANDLE USBOutHandle = 0;
USB_HANDLE USBInHandle = 0;
BOOL blinkStatusValid = FLAG_TRUE;

unsigned int temp=0;

// PIC18F25K50 configuration for the WFF Generic HID test device
// Internal 16 MHz Osc
#pragma config PLLSEL = PLL3X       // PLL Selection (3x clock multiplier) => 3 x 16 = 48 MHz
#pragma config CFGPLLEN = ON        // PLL Enable Configuration bit (PLL Enabled)
#pragma config CPUDIV = NOCLKDIV    // 1:1 mode (for 48MHz CPU)
#pragma config LS48MHZ = SYS48X8    // System chock at 48MHZ, USB divider is set to 8

// CONFIG1H
#pragma config FOSC = INTOSCIO      // Oscillator Selection (Internal oscillator)
#pragma config PCLKEN = ON          // Primary Oscillator Shutdown (Primary oscillator enabled)
#pragma config FCMEN = OFF          // Fail-Safe Clock Monitor (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF           // Internal/External Oscillator Switchover (Oscillator Switchover mode disabled)

// CONFIG2L
#pragma config nPWRTEN = OFF        // Power-up Timer Enable (Power up timer disabled)
#pragma config BOREN = SBORDIS      // Brown-out Reset Enable (BOR enabled in hardware (SBOREN is ignored))
#pragma config BORV = 190           // Brown-out Reset Voltage (BOR set to 1.9V nominal)
#pragma config nLPBOR = OFF         // Low-Power Brown-out Reset (Low-Power Brown-out Reset disabled)

// CONFIG2H
#pragma config WDTEN = OFF         // Watchdog Timer Enable bits (WDT controlled by firmware (SWDTEN enabled))
#pragma config WDTPS = 32768        // Watchdog Timer Postscaler (1:32768)

// CONFIG3H
#pragma config CCP2MX = RC1         // CCP2 MUX bit (CCP2 input/output is multiplexed with RC1)
#pragma config PBADEN = OFF         // PORTB A/D Enable bit (PORTB<5:0> pins are configured as digital I/O on Reset)
#pragma config T3CMX = RB5          // Timer3 Clock Input MUX bit (T3CKI function is on RC0)
#pragma config SDOMX = RC7          // SDO Output MUX bit (SDO function is on RC7)
#pragma config MCLRE = OFF           // Master Clear Reset Pin Enable (MCLR pin enabled; RE3 input disabled)

// CONFIG4L
#pragma config STVREN = ON          // Stack Full/Underflow Reset (Stack full/underflow will cause Reset)
#pragma config LVP = OFF             // Single-Supply ICSP Enable bit (Single-Supply ICSP enabled if MCLRE is also 1)
#pragma config XINST = OFF          // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled)
//#pragma config ICPRT = OFF                  // ICP

// CONFIG5L
#pragma config CP0 = OFF            // Block 0 Code Protect (Block 0 is not code-protected)
#pragma config CP1 = OFF            // Block 1 Code Protect (Block 1 is not code-protected)
#pragma config CP2 = OFF            // Block 2 Code Protect (Block 2 is not code-protected)
#pragma config CP3 = OFF            // Block 3 Code Protect (Block 3 is not code-protected)

// CONFIG5H
#pragma config CPB = OFF            // Boot Block Code Protect (Boot block is not code-protected)
#pragma config CPD = OFF            // Data EEPROM Code Protect (Data EEPROM is not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF           // Block 0 Write Protect (Block 0 (0800-1FFFh) is not write-protected)
#pragma config WRT1 = OFF           // Block 1 Write Protect (Block 1 (2000-3FFFh) is not write-protected)
#pragma config WRT2 = OFF           // Block 2 Write Protect (Block 2 (04000-5FFFh) is not write-protected)
#pragma config WRT3 = OFF           // Block 3 Write Protect (Block 3 (06000-7FFFh) is not write-protected)

// CONFIG6H
#pragma config WRTC = OFF           // Configuration Registers Write Protect (Configuration registers (300000-3000FFh) are not write-protected)
#pragma config WRTB = OFF           // Boot Block Write Protect (Boot block (0000-7FFh) is not write-protected)
#pragma config WRTD = OFF           // Data EEPROM Write Protect (Data EEPROM is not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF          // Block 0 Table Read Protect (Block 0 is not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF          // Block 1 Table Read Protect (Block 1 is not protected from table reads executed in other blocks)
#pragma config EBTR2 = OFF          // Block 2 Table Read Protect (Block 2 is not protected from table reads executed in other blocks)
#pragma config EBTR3 = OFF          // Block 3 Table Read Protect (Block 3 is not protected from table reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF          // Boot Block Table Read Protect (Boot block is not protected from table reads executed in other blocks)

// Private function prototypes
static void initialisePic(void);
void processUsbCommands(void);
void applicationInit(void);
void USBCBSendResume(void);
void highPriorityISRCode();
void lowPriorityISRCode();
void strobe_LED(unsigned char, unsigned char, unsigned char, unsigned char);
void timer1_isr();
void update_pattern(void);
void next_pattern(void);
void back_pattern(void);
void handle_push_button(void);
unsigned int brand(void);

// Remap vectors for compatibilty with Microchip USB boot loaders
#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018
#elif defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)	
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x800
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x808
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x818
#else	
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x00
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x08
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x18
#endif

#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER) || defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
	extern void _startup (void);
	#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
	void _reset (void)
	{
	    _asm goto _startup _endasm
	}
#endif

#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS
void Remapped_High_ISR (void)
{
     _asm goto highPriorityISRCode _endasm
}

#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
void Remapped_Low_ISR (void)
{
     _asm goto lowPriorityISRCode _endasm
}

#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER) || defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
#pragma code HIGH_INTERRUPT_VECTOR = 0x08
void High_ISR (void)
{
     _asm goto REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS _endasm
}

#pragma code LOW_INTERRUPT_VECTOR = 0x18
void Low_ISR (void)
{
     _asm goto REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS _endasm
}
#endif

#pragma code

// High-priority ISR handling function
#pragma interrupt highPriorityISRCode

//
// This is the "C" code that generated 1098 lines of assembly. I wrote it in 72 instructions
// for the interupt. I just want to keed the old code so I can whine about it.
//
//         intensity = (intensity + 1) % 8;
//         if (intensity == 0) { 
//           strobe = (strobe + 8) % 64;
//         }
//         red_out = 0x00;
//         green_out = 0x00;
//         blue_out = 0x00;
//         if (led_data[strobe+0].red >intensity) { red_out += 0b00000001;}
//         if (led_data[strobe+0].green >intensity) { green_out += 0b00000001;}
//         if (led_data[strobe+0].blue >intensity) { blue_out += 0b00000001;}

//         if (led_data[strobe+1].red >intensity) { red_out += 0b00000010;}
//         if (led_data[strobe+1].green >intensity) { green_out += 0b00000010;}
//         if (led_data[strobe+1].blue >intensity) { blue_out += 0b00000010;}

//         if (led_data[strobe+2].red >intensity) { red_out += 0b00000100;}
//         if (led_data[strobe+2].green >intensity) { green_out += 0b00000100;}
//         if (led_data[strobe+2].blue >intensity) { blue_out += 0b00000100;}

//         if (led_data[strobe+3].red >intensity) { red_out += 0b00001000;}
//         if (led_data[strobe+3].green >intensity) { green_out += 0b00001000;}
//         if (led_data[strobe+3].blue >intensity) { blue_out += 0b00001000;}

//         if (led_data[strobe+4].red >intensity) { red_out += 0b00010000;}
//         if (led_data[strobe+4].green >intensity) { green_out += 0b00010000;}
//         if (led_data[strobe+4].blue >intensity) { blue_out += 0b00010000;}

//         if (led_data[strobe+5].red >intensity) { red_out += 0b00100000;}
//         if (led_data[strobe+5].green >intensity) { green_out += 0b00100000;}
//         if (led_data[strobe+5].blue >intensity) { blue_out += 0b00100000;}

//         if (led_data[strobe+6].red >intensity) { red_out += 0b01000000;}
//         if (led_data[strobe+6].green >intensity) { green_out += 0b01000000;}
//         if (led_data[strobe+6].blue >intensity) { blue_out += 0b01000000;}

//         if (led_data[strobe+7].red >intensity) { red_out += 0b10000000;}
//         if (led_data[strobe+7].green >intensity) { green_out += 0b10000000;}
//         if (led_data[strobe+7].blue >intensity) { blue_out += 0b10000000;}





void highPriorityISRCode()
{

   #if defined(USB_INTERRUPT)
		// Perform USB device tasks
		USBDeviceTasks();
   #endif

// Application specific high-priority ISR code goes here
   if (PIR2bits.TMR3IF){  // Interrupt Check	
      PIR2bits.TMR3IF = 0;
      
   _asm
      movlw 0xF7  
      movwf TMR3H, 0 // ACCESS
      clrf  TMR3L, 0 // ACCESS

//    our data is on page 1
      movlw 0x01
      movwf BSR,0 //ACCESS

      incf intensity, 0, 1   // W, BANKED    //    W = intensity + 1
      andlw 0x07                             //    W = W % 8
      movwf intensity, 1 // BANKED           //    intensity = W

      bnz   led_strobe1                      //    if (intensity) == 0 then 
 
      movlw 0x08
      addwf strobe, 0,1 // W, BANKED         //     W = strobe + 8

      andlw 0x3f                             //     W = W % 64
      movwf strobe, 1 // BANKED              //     strobe = W

      bnz   led_strobe1                      //    if (strobe) == 0 then

      incf idelay, 1, 1   // F, BANKED       //    W = idelay + 1
      andlw 0x07                             //    W = W % 8
      movwf idelay, 1 // BANKED              //    idelay = W

      bnz   led_strobe1                      //    if (idelay) == 0 then

      clrf  wait_timer, 1 // BANKED          //    slow pattern clock

led_strobe1:


      movf strobe, 0,1 // W, BANKED         //     W = strobe
      mullw 0x03                            //     PROD[HL] = W * 3;

//     FRS0L (pointer0 low) = led_data low + PRODL (multiply low)
//      movlw low led_data
      movlw led_data // the "low" is not working!

      addwf PRODL, 0,0 // W, ACCESS
      movwf FSR0L, 0 // ACCESS     

//     FRS0H (pointer0 hi) + led_data hi + PRODH (multiply low) + Carry
//      movlw high led_data
      movlw 0x02   // the "high" is not working!, led_data is at 0x0200
      addwfc PRODH, 0,0 // W, ACCESS
      movwf FSR0H, 0 // ACCESS
   
      clrf  red_out, 1 // BANKED
      clrf  green_out,1 // BANKED
      clrf  blue_out,1 // BANKED

//     do this 24 times
//     contents of pointer0 -> W, pointer0++
//     compair intensity with W, skip greater then 
//     set bit or skip

      incf  intensity, 0, 1 // W BANKED
      cpfslt POSTINC0, 0  // ACCESS
      bsf   red_out,0, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   green_out,0, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   blue_out,0, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   red_out,1, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   green_out,1, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   blue_out,1, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   red_out,2, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   green_out,2, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   blue_out,2, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   red_out,3, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   green_out,3, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   blue_out,3, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   red_out,4, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   green_out,4, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   blue_out,4, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   red_out,5, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   green_out,5, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   blue_out,5, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   red_out,6, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   green_out,6, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   blue_out,6, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   red_out,7, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   green_out,7, 1 // BANKED

      cpfslt POSTINC0, 0  // ACCESS
      bsf   blue_out,7, 1 // BANKED

   _endasm

         strobe_LED(red_out, green_out, blue_out, (strobe>>3) );
   }

}

// Low-priority ISR handling function
#pragma interruptlow lowPriorityISRCode
void lowPriorityISRCode()
{
	// Application specific low-priority ISR code goes here
    wait_timer = FLAG_FALSE;
}

// String for creating debug messages
char debugString[80];

BOOL run = FLAG_TRUE;
BOOL demo_mode = FLAG_TRUE;

unsigned char p_intensity=0,table_type=0,cycle_count =0,pattern_speed=0;
char p_table=0;
unsigned int p_count=0;
BOOL p_up_down=FLAG_FALSE;

unsigned char demo_loops;
unsigned char old_button = 0;


// Main program entry point
void main(void)
{   
    unsigned char delay=0;

	// Initialise and configure the PIC ready to go
    initialisePic();

    // Show that we are up and running
//    mStatusLED0_on();

	// If we are running in interrupt mode attempt to attach the USB device
    #if defined(USB_INTERRUPT)
        USBDeviceAttach();
    #endif
	
	// Initialise the debug log functions
    debugInitialise();
	
	sprintf(debugString, "2DKits: 4x4x4 RGB cube 24K50 v1409");
	debugOut(debugString);
	
	sprintf(debugString, "USB Device Initialised.");
	debugOut(debugString);
	
    demo_mode = Read_b_eep(0x00);

    demo_loops = pattern_demo_loops[p_table]; // set 1st table delay


	// Main processing loop
    while(1)
    {
        #if defined(USB_POLLING)
			// If we are in polling mode the USB device tasks must be processed here
			// (otherwise the interrupt is performing this task)
	        USBDeviceTasks();
        #endif
    	
    	// Process USB Commands
        processUsbCommands();  

        // Note: Other application specific actions can be placed here
         if ((wait_timer == 0) && (run == TRUE)) { 
            wait_timer = 1;
            delay++;
#ifndef DEBUG
            if ((!sw0) || (!sw1) || old_button) {  
                handle_push_button();
            } else {
#endif
               if (delay > pattern_speed) {
                  delay = 0; 
                  update_pattern();
               }
#ifndef DEBUG
            }
#endif
         }
    } // while   
}

unsigned char fader_cycle=0;
// unsigned char button_delay=0

#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))

unsigned int up_down( unsigned char change, unsigned char data) {

  if (change) {
      return(MIN(8,data+1));
  } else {
      return(MAX(0,(signed char) data-1));
  }
}

void update_pattern() {

   unsigned char i, j, temp_in;
   static unsigned char temp_red[8],temp_green[8],temp_blue[8];
   unsigned int led_row;

// pull extra info bytes off the front of the pattern;

   if (p_count == 0) {
      table_type = patterns[p_table][p_count++];
      pattern_speed = patterns[p_table][p_count++];
   }

   switch (table_type) {
   case 1:
      for (i=0;i<8;i++) {
         temp_in = patterns[p_table][p_count];
         led_row = i*8;
         led_data[led_row+0].red = (temp_in & 0x80) ? (0x07) : (0x00);
         led_data[led_row+1].red = (temp_in & 0x40) ? (0x07) : (0x00);
         led_data[led_row+2].red = (temp_in & 0x20) ? (0x07) : (0x00);
         led_data[led_row+3].red = (temp_in & 0x10) ? (0x07) : (0x00);
         led_data[led_row+4].red = (temp_in & 0x08) ? (0x07) : (0x00);
         led_data[led_row+5].red = (temp_in & 0x04) ? (0x07) : (0x00);
         led_data[led_row+6].red = (temp_in & 0x02) ? (0x07) : (0x00);
         led_data[led_row+7].red = (temp_in & 0x01) ? (0x07) : (0x00);
         p_count++;
         temp_in = patterns[p_table][p_count];
         led_data[led_row+0].green = (temp_in & 0x80) ? (0x07) : (0x00);
         led_data[led_row+1].green = (temp_in & 0x40) ? (0x07) : (0x00);
         led_data[led_row+2].green = (temp_in & 0x20) ? (0x07) : (0x00);
         led_data[led_row+3].green = (temp_in & 0x10) ? (0x07) : (0x00);
         led_data[led_row+4].green = (temp_in & 0x08) ? (0x07) : (0x00);
         led_data[led_row+5].green = (temp_in & 0x04) ? (0x07) : (0x00);
         led_data[led_row+6].green = (temp_in & 0x02) ? (0x07) : (0x00);
         led_data[led_row+7].green = (temp_in & 0x01) ? (0x07) : (0x00);
         p_count++;
         temp_in = patterns[p_table][p_count];
         led_data[led_row+0].blue = (temp_in & 0x80) ? (0x07) : (0x00);
         led_data[led_row+1].blue = (temp_in & 0x40) ? (0x07) : (0x00);
         led_data[led_row+2].blue = (temp_in & 0x20) ? (0x07) : (0x00);
         led_data[led_row+3].blue = (temp_in & 0x10) ? (0x07) : (0x00);
         led_data[led_row+4].blue = (temp_in & 0x08) ? (0x07) : (0x00);
         led_data[led_row+5].blue = (temp_in & 0x04) ? (0x07) : (0x00);
         led_data[led_row+6].blue = (temp_in & 0x02) ? (0x07) : (0x00);
         led_data[led_row+7].blue = (temp_in & 0x01) ? (0x07) : (0x00);
         p_count++;
      }
      break;
   case 2:
   case 3:
      if (fader_cycle == 0) {
         if (table_type == 3) {srand(18762);}
         for (i=0;i<8;i++) {
            if (table_type == 2) {
            temp_red[i] = patterns[p_table][p_count++];
            temp_green[i] = patterns[p_table][p_count++];
            temp_blue[i] = patterns[p_table][p_count++];
            } else if (table_type == 3) {
            temp_red[i] = brand();
            temp_green[i] = brand();
            temp_blue[i] = brand();
            }
         }
         cycle_count = patterns[p_table][p_count++];
      }
      for (i=0;i<8;i++) {
         led_row = i*8;    
         led_data[led_row+0].red = up_down((temp_red[i] & 0x80),led_data[led_row+0].red);
         led_data[led_row+1].red = up_down((temp_red[i] & 0x40),led_data[led_row+1].red);
         led_data[led_row+2].red = up_down((temp_red[i] & 0x20),led_data[led_row+2].red);
         led_data[led_row+3].red = up_down((temp_red[i] & 0x10),led_data[led_row+3].red); 
         led_data[led_row+4].red = up_down((temp_red[i] & 0x08),led_data[led_row+4].red);
         led_data[led_row+5].red = up_down((temp_red[i] & 0x04),led_data[led_row+5].red);
         led_data[led_row+6].red = up_down((temp_red[i] & 0x02),led_data[led_row+6].red);
         led_data[led_row+7].red = up_down((temp_red[i] & 0x01),led_data[led_row+7].red);
         led_data[led_row+0].green = up_down((temp_green[i] & 0x80),led_data[led_row+0].green);
         led_data[led_row+1].green = up_down((temp_green[i] & 0x40),led_data[led_row+1].green);
         led_data[led_row+2].green = up_down((temp_green[i] & 0x20),led_data[led_row+2].green);
         led_data[led_row+3].green = up_down((temp_green[i] & 0x10),led_data[led_row+3].green);
         led_data[led_row+4].green = up_down((temp_green[i] & 0x08),led_data[led_row+4].green);
         led_data[led_row+5].green = up_down((temp_green[i] & 0x04),led_data[led_row+5].green);
         led_data[led_row+6].green = up_down((temp_green[i] & 0x02),led_data[led_row+6].green);
         led_data[led_row+7].green = up_down((temp_green[i] & 0x01),led_data[led_row+7].green);
         led_data[led_row+0].blue = up_down((temp_blue[i] & 0x80),led_data[led_row+0].blue);
         led_data[led_row+1].blue = up_down((temp_blue[i] & 0x40),led_data[led_row+1].blue);
         led_data[led_row+2].blue = up_down((temp_blue[i] & 0x20),led_data[led_row+2].blue);
         led_data[led_row+3].blue = up_down((temp_blue[i] & 0x10),led_data[led_row+3].blue);
         led_data[led_row+4].blue = up_down((temp_blue[i] & 0x08),led_data[led_row+4].blue);
         led_data[led_row+5].blue = up_down((temp_blue[i] & 0x04),led_data[led_row+5].blue);
         led_data[led_row+6].blue = up_down((temp_blue[i] & 0x02),led_data[led_row+6].blue);
         led_data[led_row+7].blue = up_down((temp_blue[i] & 0x01),led_data[led_row+7].blue);
      }
      fader_cycle = (fader_cycle < cycle_count) ? (fader_cycle+1) : (0);
      break;
case 4:
      if (fader_cycle == 0) {
         for (i=0;i<8;i++) {
            temp_red[i] = 0;
            temp_green[i] = 0;
            temp_blue[i] = 0;
            for (j=0;j<8;j++) {
                switch (( brand()>>4) % 4) {
                case 0:
                   break;
                case 1:
                   temp_red[i] = temp_red[i] + (1<<j);
                   break;
                case 2:
                   temp_green[i] |= (1<<j);
                   break;
                case 3:
                   temp_blue[i] |= (1<<j);
                   break;
                }
            }
         }
         i = brand(); //without this you get the same
         cycle_count = patterns[p_table][p_count++];
      }
      for (i=0;i<8;i++) {
         led_row = i*8;    
         led_data[led_row+0].red = up_down((temp_red[i] & 0x80),led_data[led_row+0].red);
         led_data[led_row+1].red = up_down((temp_red[i] & 0x40),led_data[led_row+1].red);
         led_data[led_row+2].red = up_down((temp_red[i] & 0x20),led_data[led_row+2].red);
         led_data[led_row+3].red = up_down((temp_red[i] & 0x10),led_data[led_row+3].red); 
         led_data[led_row+4].red = up_down((temp_red[i] & 0x08),led_data[led_row+4].red);
         led_data[led_row+5].red = up_down((temp_red[i] & 0x04),led_data[led_row+5].red);
         led_data[led_row+6].red = up_down((temp_red[i] & 0x02),led_data[led_row+6].red);
         led_data[led_row+7].red = up_down((temp_red[i] & 0x01),led_data[led_row+7].red);
         led_data[led_row+0].green = up_down((temp_green[i] & 0x80),led_data[led_row+0].green);
         led_data[led_row+1].green = up_down((temp_green[i] & 0x40),led_data[led_row+1].green);
         led_data[led_row+2].green = up_down((temp_green[i] & 0x20),led_data[led_row+2].green);
         led_data[led_row+3].green = up_down((temp_green[i] & 0x10),led_data[led_row+3].green);
         led_data[led_row+4].green = up_down((temp_green[i] & 0x08),led_data[led_row+4].green);
         led_data[led_row+5].green = up_down((temp_green[i] & 0x04),led_data[led_row+5].green);
         led_data[led_row+6].green = up_down((temp_green[i] & 0x02),led_data[led_row+6].green);
         led_data[led_row+7].green = up_down((temp_green[i] & 0x01),led_data[led_row+7].green);
         led_data[led_row+0].blue = up_down((temp_blue[i] & 0x80),led_data[led_row+0].blue);
         led_data[led_row+1].blue = up_down((temp_blue[i] & 0x40),led_data[led_row+1].blue);
         led_data[led_row+2].blue = up_down((temp_blue[i] & 0x20),led_data[led_row+2].blue);
         led_data[led_row+3].blue = up_down((temp_blue[i] & 0x10),led_data[led_row+3].blue);
         led_data[led_row+4].blue = up_down((temp_blue[i] & 0x08),led_data[led_row+4].blue);
         led_data[led_row+5].blue = up_down((temp_blue[i] & 0x04),led_data[led_row+5].blue);
         led_data[led_row+6].blue = up_down((temp_blue[i] & 0x02),led_data[led_row+6].blue);
         led_data[led_row+7].blue = up_down((temp_blue[i] & 0x01),led_data[led_row+7].blue);
      }
      fader_cycle = (fader_cycle < cycle_count) ? (fader_cycle+1) : (0);
      break;
   }

   if (p_count >= pattern_size[p_table]) {
      p_count = 0;
      demo_loops = demo_loops - 1;
      if ((demo_loops == 0) & (demo_mode == TRUE)) {
         next_pattern();
      }
   }
}

void handle_push_button() {

#ifdef S4X4
#define DEMO_MODE_LED 8
#else
#define DEMO_MODE_LED 32
#endif

unsigned char i, temp;
   if (old_button == 0) {
      for (i=0;i<4;i++) {
         led_data_save[i] = led_data[i];      // save the 4 LED states
         led_data[i].red = 0x00;
         led_data[i].green = 0x00;
         led_data[i].blue = 0x00;
      }
      led_data_save[4] = led_data[DEMO_MODE_LED];
      led_data[DEMO_MODE_LED].red = 0x00;
      led_data[DEMO_MODE_LED].green = 0x00;
      led_data[DEMO_MODE_LED].blue = 0x00;
   }

   temp = p_table+1;

   led_data[DEMO_MODE_LED].blue = (demo_mode) ? (0xff) : (0x00);  // display the table number  
   led_data[0].red = (temp & 0x01) ? (0xff) : (0x00);  // display the table number
   led_data[1].red = (temp & 0x02) ? (0xff) : (0x00);
   led_data[2].red = (temp & 0x04) ? (0xff) : (0x00);
   led_data[3].red = (temp & 0x08) ? (0xff) : (0x00);
  
   old_button++;

#define BUTTON_DELAY 64
    mInitAllSwitches()   //DDF Slam the direction
#ifndef DEBUG
   if ((!sw0) & (!sw1)) {
      if (old_button > BUTTON_DELAY) {
         demo_mode = !demo_mode;
         Busy_eep();
         Write_b_eep(0x00, demo_mode);
         old_button = 1;
      }
   } else if (!sw0) {
      if (old_button > BUTTON_DELAY) {
         next_pattern();
         old_button = 1;
      }
   } else if (!sw1) {
      if (old_button > BUTTON_DELAY) {
         back_pattern();
         old_button = 1;
      }
   } else {
      for (i=0;i++;i<4) {
         led_data[i] = led_data_save[i];  // no button, restore the 4 LEDs state.
      }
      led_data[DEMO_MODE_LED] = led_data_save[4];  // no button, restore the 4 LEDs state.
      old_button = 0;
      sprintf(debugString, "pattern %d %d", p_table, demo_mode);
	  debugOut(debugString);
   }
#endif
}

void next_pattern(void) {
      p_table = (p_table >= table_count) ? (0):(p_table+1);
      p_count = 0;
      demo_loops = pattern_demo_loops[p_table];
      sprintf(debugString, "Up to pattern %d %d", p_table, demo_loops);
 	  debugOut(debugString);
}

void back_pattern(void) {
     p_table = (p_table < 1) ? (table_count):(p_table-1);
     p_count = 0;
     demo_loops = pattern_demo_loops[p_table];
     sprintf(debugString, "Down to pattern %d %d", p_table, demo_loops);
 	 debugOut(debugString);
}

unsigned brand() {

    static unsigned int lfsr = 0xACE1 ;
    unsigned int bit;

    bit  = ((lfsr >> 0) ^ (lfsr >> 2) ^ (lfsr >> 3) ^ (lfsr >> 5) ) & 1;
    lfsr =  (lfsr >> 1) | (bit << 15);

    return lfsr;
}

// Initialise the PIC
static void initialisePic(void)
{
    unsigned char dummySPIdata;
    unsigned char i;

   // Fix the settings needed for USB
    ACTCONbits.ACTEN = 0; // ACT module is enabled, updates to OSCTUNE are exclusive to the ACT module.
    ACTCONbits.ACTSRC = 1; //Active Clock Tuning Source Selection bit
    ACTCONbits.ACTUD = 0; // Updates to the OSCTUNE register from ACT module are enabled.
    OSCCONbits.IRCF = 7; // HFINTOSC  (16 MHz)

    // PIC port set up --------------------------------------------------------

	// Default all pins to digital
    ADCON1 = 0x0F;                                                                                                                                                                                                                                                                                             

	// Configure ports as inputs (1) or outputs(0)
	TRISA = 0b00110000;
	TRISB = 0b00000000;
	TRISC = 0b00000000;


	// Clear all ports
	PORTA = 0b00000000;
	PORTB = 0b00000000;
	PORTC = 0b00000000;

	// If you have a VBUS sense pin (for self-powered devices when you
	// want to detect if the USB host is connected) you have to specify
	// your input pin in HardwareProfile.h
    #if defined(USE_USB_BUS_SENSE_IO)
    	tris_usb_bus_sense = INPUT_PIN;
    #endif
    
    // In the case of a device which can be both self-powered and bus-powered
    // the device must respond correctly to a GetStatus (device) request and
    // tell the host how it is currently powered.
    //
    // To do this you must device a pin which is high when self powered and low
    // when bus powered and define this in HardwareProfile.h
    #if defined(USE_SELF_POWER_SENSE_IO)
    	tris_self_power = INPUT_PIN;
    #endif

    // Application specific initialisation
    applicationInit();
    
    // Initialise the USB device
    USBDeviceInit();

    // setup the LED drivers and SPI buss
 #define LED_enable 		LATBbits.LATB3
 #define LED_enable_dir		TRISBbits.TRISB3
 #define LED_latch   		LATBbits.LATB2
 #define LED_latch_dir		TRISBbits.TRISB2

 #define SPI_DataOut 		LATCbits.LATC7
 #define SPI_DataIn			POATBbits.RB0
 #define SPI_Clk 			LATBbits.LATB1
 #define SPI_DataOut_Dir 	TRISCbits.TRISC7
 #define SPI_DataIn_Dir 	TRISBbits.TRISB0
 #define SPI_Clk_Dir 		TRISBbits.TRISB1


 #define LED_row0 		    LATAbits.LATA0
 #define LED_row0_dir		TRISAbits.TRISA0
 #define LED_row1 		    LATAbits.LATA1
 #define LED_row1_dir		TRISAbits.TRISA1
 #define LED_row2 		    LATAbits.LATA2
 #define LED_row2_dir		TRISAbits.TRISA2
 #define LED_row3 		    LATAbits.LATA3
 #define LED_row3_dir		TRISAbits.TRISA3
 #define LED_row4 		    LATBbits.LATB4
 #define LED_row4_dir		TRISBbits.TRISB4
 #define LED_row5 		    LATBbits.LATB5
 #define LED_row5_dir		TRISBbits.TRISB5
 #define LED_row6 		    LATBbits.LATB6
 #define LED_row6_dir		TRISBbits.TRISB6
 #define LED_row7 		    LATBbits.LATB7
 #define LED_row7_dir		TRISBbits.TRISB7

	//Configure I/O for LED and SPI

	SPI_DataOut_Dir 	=0;
	SPI_DataIn_Dir 		=1;
	SPI_Clk_Dir 		=0;

    LED_enable_dir    =0;  //output
    LED_latch_dir    =0;  //output

    LED_row0_dir     =0;   //output
    LED_row1_dir     =0;   //output
    LED_row2_dir     =0;   //output
    LED_row3_dir     =0;   //output
    LED_row4_dir     =0;   //output
    LED_row5_dir     =0;   //output
    LED_row6_dir     =0;   //output
    LED_row7_dir     =0;   //output

    LED_row0         =0;   //off
    LED_row1         =0;   //off
    LED_row2         =0;   //off
    LED_row3         =0;   //off
    LED_row4         =0;   //off
    LED_row5         =0;   //off
    LED_row6         =0;   //off
    LED_row7         =0;   //off

    LED_enable        =0;  // set low to turn on drivers
    LED_latch        =0;  // latch off

    OpenSPI(SPI_FOSC_4 & SLV_SSOFF, MODE_00, SMPMID);

//  OpenTimer3
    T3CON = 0b00000111;
    T3GCON = 0b00000000;
/*
    OpenTimer3(  TIMER_INT_ON &                // Interrupt enabled 
                 T3_16BIT_RW &                  //set timer1 as 16 bit registers 
                 T3_SOURCE_FOSC &               //choose internal clock source (TOSC) 
                 T3_PS_1_1 &                   // Prescale Value: 1:8 
                 T3_SYNC_EXT_OFF,               //Dont sync external clock input
                 0 
              ); 
*/

   for (i=0;i<64;i++) {
      led_data[i].red = 7;
      led_data[i].green = 0;
      led_data[i].blue = 0;
   }


	RCONbits.IPEN=1;     //Enable priority levels on interrupts
	RCONbits.SBOREN=0;   //Disable BOR

    IPR2bits.TMR3IP = 1; //Timer3 interrupt priority hi 
    T3CONbits.TMR3ON = 1;	/* Enable the timer3 */
    PIE2bits.TMR3IE = 1;	/* Enable timer3 interrupt */	
    INTCONbits.GIEH = 1; //enable hi pri interrupts



    INTCON2bits.RBPU = 0;
    ANSELA = 0;           // configure for digital IO

    WPUB = 0;                // all Port B pull-ups disabled 



}

void strobe_LED(unsigned char red, unsigned char green, unsigned char blue, unsigned char row )
{
static unsigned char old_row = 0;
//  write 24 bits of column data via SPI buss
#ifndef DEBUG
    WriteSPI(green); // all green on
    WriteSPI(red); // all red on
#endif

// I turn off the rows, then send the blue to stop a pre-ghost issue I saw.
// My guess is the Source Driver chip needs some time to turn off before I latch
// the next column.

//  all rows off 
if (old_row!=row) {
    LED_row0         =0;  // why is this command not working?
    LED_row1         =0;
    LED_row2         =0;
    LED_row3         =0;
    LED_row4         =0;
    LED_row5         =0;
    LED_row6         =0;
    LED_row7         =0;
    LED_row0         =0;  // I have no idea why I have to do it a 2nd time!!
}
#ifndef DEBUG
    WriteSPI(blue); // all blue on
#endif

//  latch the 24 bits of column data
    LED_latch        =1;  // latch the LED data
    LED_latch        =0;

if (old_row!=row) {
switch (row) {
       case 0:
          LED_row0         =1;   
          break;
       case 1:
          LED_row1         =1; 
          break;
       case 2:
          LED_row2         =1;
          break;
       case 3:
          LED_row3         =1;
          break;
       case 4:
          LED_row4         =1;
          break;
       case 5:
          LED_row5         =1;
          break;
       case 6:
          LED_row6         =1;
          break;
       case 7:
          LED_row7         =1;
          break;
       default:
          break;
   	}   
   }
   old_row = row;
}

// Application specific device initialisation
void applicationInit(void)
{
	// Initialise the status LEDs
	mInitStatusLeds();
	
	// Initialise the switch
	mInitAllSwitches();
	
    // Initialize the variable holding the USB handle for the last transmission
    USBOutHandle = 0;
    USBInHandle = 0;
}

// Process USB commands
void processUsbCommands(void)
{   
    // Check if we are in the configured state; otherwise just return
    if((USBDeviceState < CONFIGURED_STATE) || (USBSuspendControl == 1))
    {
	    // We are not configured
	    return;
	}

	// Check if data was received from the host.
    if(!HIDRxHandleBusy(USBOutHandle))
    {   
		// Command mode    
        switch(ReceivedDataBuffer[0])
		{
			case 0x10:	// Debug information request from host
				// Copy any waiting debug text to the send data buffer
				copyDebugToSendBuffer((BYTE*)&ToSendDataBuffer[0]);
			        
		        // Transmit the response to the host
                if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
				}
				break;
				
			// Place application specific commands here:
			
            case 0x80:  // Toggle the LED
            	sprintf(debugString, "Toggle LED %d", strobe);
				debugOut(debugString);

				// Toggle the LED0
				mStatusLED0_Toggle();
                strobe_LED(0x55, 0xAA, 0x0F, 1 );
                
                run = !run;
            	break;
            	
            case 0x81:  // Read the push switch status
				ToSendDataBuffer[0] = sw0 + (sw1<<1);

				// Transmit the response to the host
                if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
				}
            	break;
            	
            case 0x82:  // Read the LED status
				// Get the LED state and put it in the send buffer
				ToSendDataBuffer[0] = mStatusLED0_Get();
				
				// Transmit the response to the host
                if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
				}
            	break;

            case 0x83:  // RGB LED set command
				// process an RBG matrix command
		        led_data[ReceivedDataBuffer[4]].red = ReceivedDataBuffer[1];
		        led_data[ReceivedDataBuffer[4]].green = ReceivedDataBuffer[2];
		        led_data[ReceivedDataBuffer[4]].blue = ReceivedDataBuffer[3];
            	break;

            case 0x84:  // pattern change command
				// process an RBG matrix command
		        if (ReceivedDataBuffer[1] == 0) {
                   next_pattern();
                } else if (ReceivedDataBuffer[1] == 1) {
                   back_pattern();
                }
            	break;
            default:	// Unknown command received
           		break;
		}
        // Re-arm the OUT endpoint for the next packet
        USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
  	}
}

// USB Callback handling routines -----------------------------------------------------------

// Call back that is invoked when a USB suspend is detected
void USBCBSuspend(void)
{
}

// This call back is invoked when a wakeup from USB suspend is detected.
void USBCBWakeFromSuspend(void)
{
}

// The USB host sends out a SOF packet to full-speed devices every 1 ms.
void USBCB_SOF_Handler(void)
{
    // No need to clear UIRbits.SOFIF to 0 here. Callback caller is already doing that.
}

// The purpose of this callback is mainly for debugging during development.
// Check UEIR to see which error causes the interrupt.
void USBCBErrorHandler(void)
{
    // No need to clear UEIR to 0 here.
    // Callback caller is already doing that.
}

// Check other requests callback
void USBCBCheckOtherReq(void)
{
    USBCheckHIDRequest();
}

// Callback function is called when a SETUP, bRequest: SET_DESCRIPTOR request arrives.
void USBCBStdSetDscHandler(void)
{
    // You must claim session ownership if supporting this request
}

//This function is called when the device becomes initialized
void USBCBInitEP(void)
{
    // Enable the HID endpoint
    USBEnableEndpoint(HID_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
    
    // Re-arm the OUT endpoint for the next packet
    USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
}

// Send resume call-back
void USBCBSendResume(void)
{
    static WORD delay_count;
    
    // Verify that the host has armed us to perform remote wakeup.
    if(USBGetRemoteWakeupStatus() == FLAG_TRUE) 
    {
        // Verify that the USB bus is suspended (before we send remote wakeup signalling).
        if(USBIsBusSuspended() == FLAG_TRUE)
        {
            USBMaskInterrupts();
            
            // Bring the clock speed up to normal running state
            USBCBWakeFromSuspend();
            USBSuspendControl = 0; 
            USBBusIsSuspended = FLAG_FALSE;

            // Section 7.1.7.7 of the USB 2.0 specifications indicates a USB
            // device must continuously see 5ms+ of idle on the bus, before it sends
            // remote wakeup signalling.  One way to be certain that this parameter
            // gets met, is to add a 2ms+ blocking delay here (2ms plus at 
            // least 3ms from bus idle to USBIsBusSuspended() == FLAG_TRUE, yeilds
            // 5ms+ total delay since start of idle).
            delay_count = 3600U;        
            do
            {
                delay_count--;
            } while(delay_count);
            
            // Start RESUME signaling for 1-13 ms
            USBResumeControl = 1;
            delay_count = 1800U;
            do
            {
                delay_count--;
            } while(delay_count);
            USBResumeControl = 0;

            USBUnmaskInterrupts();
        }
    }
}

// USB callback function handler
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch(event)
    {
        case EVENT_TRANSFER:
            // Application callback tasks and functions go here
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        default:
            break;
    }      
    return FLAG_TRUE; 
}

#endif
